@@ -1,7 +1,14 @@
 from __future__ import absolute_import, unicode_literals, print_function
+import os
 import ctypes
 from ctypes import POINTER, c_int, c_double, c_void_p, byref
 
+"""
+Parallelisation within cuba is not supported, because python does not know
+that the call is in parallel and writes to the same memory location, causing
+overrides. This could be overcome by using locks.
+"""
+os.environ['CUBACORES'] = '0'
 lib = ctypes.cdll.LoadLibrary('libcuba.so')
 
 NULL = ctypes.POINTER(c_int)()
@@ -18,6 +25,7 @@ NINCREASE = 500
 NBATCH = 1000
 GRIDNO = 0
 STATEFILE = NULL
+spin = NULL
 
 class BOUNDS(ctypes.Structure):
   _fields_ = ("lower", c_double), ("upper", c_double)
@@ -31,7 +39,7 @@ def Vegas(integrand, ndim, userdata=NULL,
     epsrel=EPSREL, epsabs=EPSABS, verbose=0, ncomp=1, seed=None,
     mineval=MINEVAL, maxeval=MAXEVAL, nstart=NSTART, 
     nincrease=NINCREASE, nbatch=NBATCH,
-    gridno=GRIDNO, statefile=NULL):
+    gridno=GRIDNO, statefile=NULL, nvec=1):
   """
   *nstart*: the number of integrand evaluations per iteration to start
 with.
@@ -85,11 +93,15 @@ iteration.
     seed = 0
   
   lib.Vegas(ndim, ncomp, integrand_type(integrand), userdata,
-    c_double(epsrel), c_double(epsabs), verbose, seed,
+    c_int(nvec), c_double(epsrel), c_double(epsabs), verbose, seed,
     mineval, maxeval, nstart, nincrease, nbatch,
-    gridno, statefile,
+    gridno, statefile, spin,
     byref(neval), byref(fail), integral, error, prob)
   
+  print ("looking at return values...")
+  print ("neval", neval.value)
+  print ("fail", fail.value)
+  print ("comp", comp.value)
   return dict(neval=neval.value, fail=fail.value, comp=comp.value,
     results=[{
       'integral':integral[comp], 
@@ -97,12 +109,16 @@ iteration.
       'prob':prob[comp]
       } for comp in range(ncomp)])
 
-def Suave(integrand, ndim, nnew=1000, flatness=25., userdata=NULL, 
+def Suave(integrand, ndim, nnew=1000, nmin=2, flatness=50., userdata=NULL, 
     epsrel=EPSREL, epsabs=EPSABS, verbose=0, ncomp=1, seed=None,
-    mineval=MINEVAL, maxeval=MAXEVAL):
+    mineval=MINEVAL, maxeval=MAXEVAL, statefile=NULL, nvec=1):
   """
   *nnew*: the number of new integrand evaluations in each subdivision.
   
+  *nmin*: the minimum number of samples a former pass must contribute
+  to a subregion to be considered in that region's compound integral value. Increasing
+  nmin may reduce jumps in the chi^2 value.
+  
   *flatness*: the parameter p in Eq. (1), i.e. the type of norm
   used to compute the fluctuation of a sample. This determines how prominently 'out-
   liers,' i.e. individual samples with a large fluctuation, figure in the total fluctuation,
@@ -126,8 +142,8 @@ def Suave(integrand, ndim, nnew=1000, flatness=25., userdata=NULL,
     seed = 0
   
   lib.Suave(ndim, ncomp, integrand_type(integrand), userdata,
-    c_double(epsrel), c_double(epsabs), verbose, seed,
-    mineval, maxeval, nnew, c_double(flatness),
+    c_int(nvec), c_double(epsrel), c_double(epsabs), verbose, seed,
+    mineval, maxeval, nnew, nmin, c_double(flatness), statefile, spin,
     byref(nregions), byref(neval), byref(fail), integral, error, prob)
   
   return dict(neval=neval.value, fail=fail.value, comp=comp.value, nregions=nregions.value,
@@ -143,7 +159,7 @@ def Divonne(integrand, ndim,
     mineval=MINEVAL, maxeval=MAXEVAL, ncomp=1,
     ldxgiven=None, xgiven=None, nextra=0, peakfinder=None,
     userdata=NULL, seed=None,
-    epsrel=EPSREL, epsabs=EPSABS, verbose=0):
+    epsrel=EPSREL, epsabs=EPSABS, verbose=0, statefile=NULL, nvec=1):
   """
   *key1*: determines sampling in the partitioning phase:
     key1 = 7, 9, 11, 13 selects the cubature rule of degree key1. Note that the degree-11
@@ -269,10 +285,10 @@ def Divonne(integrand, ndim,
     seed = 0
 
   lib.Divonne(ndim, ncomp, integrand_type(integrand), userdata,
-    c_double(epsrel), c_double(epsabs), verbose, seed,
+    c_int(nvec), c_double(epsrel), c_double(epsabs), verbose, seed,
     mineval, maxeval, key1, key2, key3, maxpass, 
     c_double(border), c_double(maxchisq), c_double(mindeviation), 
-    ngiven, ldxgiven, xgiven, nextra, peakfinder, 
+    ngiven, ldxgiven, xgiven, nextra, peakfinder, statefile, spin,
     byref(nregions), byref(neval), byref(fail), integral, error, prob)
   
   return dict(neval=neval.value, fail=fail.value, comp=comp.value, nregions=nregions.value,
@@ -285,7 +301,7 @@ def Divonne(integrand, ndim,
 def Cuhre(integrand, ndim, 
     key=0, mineval=MINEVAL, maxeval=MAXEVAL, ncomp=1,
     userdata=NULL, seed=None,
-    epsrel=EPSREL, epsabs=EPSABS, verbose=0):
+    epsrel=EPSREL, epsabs=EPSABS, verbose=0, statefile=NULL, nvec=1):
   """
   *key* chooses the basic integration rule:
     key = 7, 9, 11, 13 selects the cubature rule of degree key. Note that the degree-11
@@ -307,8 +323,8 @@ def Cuhre(integrand, ndim,
     seed = 0
 
   lib.Cuhre(ndim, ncomp, integrand_type(integrand), userdata,
-    c_double(epsrel), c_double(epsabs), verbose,
-    mineval, maxeval, key, 
+    c_int(nvec), c_double(epsrel), c_double(epsabs), verbose,
+    mineval, maxeval, key, statefile, spin,
     byref(nregions), byref(neval), byref(fail), integral, error, prob)
   
   return dict(neval=neval.value, fail=fail.value, comp=comp.value, nregions=nregions.value,
@@ -322,9 +338,11 @@ def demo():
   import math
 
   def Integrand(ndim, xx, ncomp, ff, userdata):
+    print("integrand called")
     x,y,z = [xx[i] for i in range(ndim.contents.value)]
     result = math.sin(x)*math.cos(y)*math.exp(z)
     ff[0] = result
+    print("integrand returning")
     return 0
     
 
